package router

import (
	"applet/app/cfg"
	"applet/app/mall/hdl"
	"applet/app/mw"
	pay "applet/app/pay/hdl"
	_ "applet/docs"
	"github.com/gin-gonic/gin"
)

//初始化路由
func Init() *gin.Engine {
	// debug, release, test 项目阶段
	mode := "release"
	if cfg.Debug {
		mode = "debug"
	}
	gin.SetMode(mode)
	//创建一个新的启动器
	r := gin.New()
	r.Use(mw.ChangeHeader)

	// 是否打印访问日志, 在非正式环境都打印
	if mode != "release" {
		r.Use(gin.Logger())
	}
	r.Use(gin.Recovery())
	// r.Use(mw.Limiter)
	//r.LoadHTMLGlob("static/html/*")

	r.GET("/favicon.ico", func(c *gin.Context) {
		c.Status(204)
	})
	r.NoRoute(func(c *gin.Context) {
		c.JSON(404, gin.H{"code": 404, "msg": "page not found", "data": []struct{}{}})
	})
	r.NoMethod(func(c *gin.Context) {
		c.JSON(405, gin.H{"code": 405, "msg": "method not allowed", "data": []struct{}{}})
	})
	r.Use(mw.Cors)
	route(r.Group("/api/v1/mall"))
	rInApi(r.Group("/inapi/mall"))
	return r
}

func route(r *gin.RouterGroup) {
	// 通用支付回调
	r.Any("/pay/callback", pay.PayCallback)
	r.Use(mw.DB) // 以下接口需要用到数据库
	{
		r.GET("/test", hdl.Demo1)
	}

	r.Use(mw.Checker) // 以下接口需要检查Header: platform
	{
	}

	r.Use(mw.AuthJWT) // 以下接口需要JWT验证
	{
		// 通用支付
		r.POST("/user/pay/:payMethod/:orderType", pay.Pay)
		// 支付状态
		r.POST("/user/paystatus/:orderType", pay.PayStatus)
	}
}

func rInApi(r *gin.RouterGroup) {
	//TODO::该分组中所有的接口，支持开放平台调用
	r.Use(mw.DB)                                                       // 以下接口需要用到数据库
	{

	}
	r.Use(mw.AuthJWT) // 以下接口需要JWT验证
	{

	}
}
